var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useCallback, useEffect, useState } from 'react';
import { closeEditor, updateCellValue } from '../../actionCreators';
import { ActionType } from '../../enums';
import { getValueByColumn, replaceValue } from '../../Utils/DataUtils';
import { addEscEnterKeyEffect } from '../../Utils/EffectUtils';
import { getValidationValue } from '../../Utils/Validation';
import CellEditorDataType from '../CellEditorDataType/CellEditorDataType';
import CellEditorValidationMessage from '../CellEditorValidationMessage/CellEditorValidationMessage';
var CellEditorState = function (props) {
    var column = props.column, rowData = props.rowData, rowKeyValue = props.rowKeyValue, dispatch = props.dispatch;
    var _a = useState(rowData), rowDataState = _a[0], changeValue = _a[1];
    var validationValue = getValidationValue(rowDataState, column);
    var onValueStateChange = function (action) {
        var newRowValue = replaceValue(rowData, column, action.value);
        changeValue(newRowValue);
    };
    var close = useCallback(function () {
        dispatch(closeEditor(rowKeyValue, column.key));
    }, [dispatch, column, rowKeyValue]);
    var closeHandler = useCallback(function () {
        if (!validationValue) {
            var newValue = getValueByColumn(rowDataState, column);
            if (getValueByColumn(rowData, column) !== newValue) {
                dispatch(updateCellValue(rowKeyValue, column.key, newValue));
            }
            close();
        }
    }, [validationValue, dispatch, close, rowDataState, column, rowData, rowKeyValue]);
    useEffect(function () {
        return addEscEnterKeyEffect(close, closeHandler);
    }, [close, closeHandler]);
    var dispatchHandler = function (action) {
        if (action.type === ActionType.CloseEditor) {
            closeHandler();
        }
        else if (action.type === ActionType.UpdateCellValue) {
            onValueStateChange(action);
        }
        else {
            dispatch(action);
        }
    };
    var stateProps = __assign(__assign({}, props), {
        dispatch: dispatchHandler,
        rowData: rowDataState,
        value: getValueByColumn(rowDataState, column),
    });
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: "ka-cell-editor " + (validationValue ? 'ka-cell-editor-validation-error' : '') },
            React.createElement(CellEditorDataType, __assign({}, stateProps)),
            validationValue && React.createElement(CellEditorValidationMessage, { message: validationValue }))));
};
export default CellEditorState;