var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
import { ActionType } from '../enums';
import { getCopyOfArrayAndInsertOrReplaceItem } from '../Utils/ArrayUtils';
import { addItemToEditableCells, removeItemFromEditableCells } from '../Utils/CellUtils';
import { replaceValue } from '../Utils/DataUtils';
import { getExpandedGroups, updateExpandedGroups } from '../Utils/GroupUtils';
import { getSortedColumns } from '../Utils/HeadRowUtils';
import { prepareTableOptions } from '../Utils/PropsUtils';
var kaReducer = function (state, action) {
    var columns = state.columns, _a = state.data, data = _a === void 0 ? [] : _a, _b = state.editableCells, editableCells = _b === void 0 ? [] : _b, groupsExpanded = state.groupsExpanded, loading = state.loading, paging = state.paging, rowKeyField = state.rowKeyField, _c = state.selectedRows, selectedRows = _c === void 0 ? [] : _c, virtualScrolling = state.virtualScrolling;
    switch (action.type) {
        case ActionType.UpdatePageIndex: {
            return __assign(__assign({}, state), { paging: __assign(__assign({}, paging), { pageIndex: action.pageIndex }) });
        }
        case ActionType.HideLoading: {
            return __assign(__assign({}, state), { loading: __assign(__assign({}, loading), { enabled: false }) });
        }
        case ActionType.ShowLoading: {
            var newLoading = __assign(__assign({}, loading), { enabled: true });
            if (action.text !== undefined) {
                newLoading.text = action.text;
            }
            return __assign(__assign({}, state), { loading: newLoading });
        }
        case ActionType.OpenEditor: {
            var newEditableCells = addItemToEditableCells(action, editableCells);
            return __assign(__assign({}, state), { editableCells: newEditableCells });
        }
        case ActionType.CloseEditor: {
            var newEditableCells = removeItemFromEditableCells(action, editableCells);
            return __assign(__assign({}, state), { editableCells: newEditableCells });
        }
        case ActionType.UpdateFilterRowValue: {
            var column = columns.find(function (c) { return c.key === action.columnKey; });
            var newColumn = __assign(__assign({}, column), { filterRowValue: action.filterRowValue });
            var newColumns = getCopyOfArrayAndInsertOrReplaceItem(newColumn, 'key', columns);
            return __assign(__assign({}, state), { columns: newColumns });
        }
        case ActionType.UpdateFilterRowOperator: {
            var column = columns.find(function (c) { return c.key === action.columnKey; });
            var newColumn = __assign(__assign({}, column), { filterRowOperator: action.filterRowOperator });
            var newColumns = getCopyOfArrayAndInsertOrReplaceItem(newColumn, 'key', columns);
            return __assign(__assign({}, state), { columns: newColumns });
        }
        case ActionType.UpdateCellValue: {
            var row = data.find(function (d) { return d[rowKeyField] === action.rowKeyValue; });
            var column = columns.find(function (c) { return c.key === action.columnKey; });
            var newRowData = replaceValue(row, column, action.value);
            var newData = getCopyOfArrayAndInsertOrReplaceItem(newRowData, rowKeyField, data);
            return __assign(__assign({}, state), { data: newData });
        }
        case ActionType.DeleteRow: {
            var newData = data.filter(function (d) { return d[rowKeyField] !== action.rowKeyValue; });
            return __assign(__assign({}, state), { data: newData });
        }
        case ActionType.SelectAllRows: {
            var newSelectedRows = data.map(function (d) { return d[rowKeyField]; });
            return __assign(__assign({}, state), { selectedRows: newSelectedRows });
        }
        case ActionType.Search: {
            return __assign(__assign({}, state), { search: action.search });
        }
        case ActionType.SelectSingleRow: {
            var newSelectedRows = [action.rowKeyValue];
            return __assign(__assign({}, state), { selectedRows: newSelectedRows });
        }
        case ActionType.DeselectAllRows:
            return __assign(__assign({}, state), { selectedRows: [] });
        case ActionType.SelectRow:
            return __assign(__assign({}, state), { selectedRows: __spreadArrays(selectedRows, [action.rowKeyValue]) });
        case ActionType.DeselectRow: {
            var newSelectedRows = __spreadArrays(selectedRows).filter(function (s) { return s !== action.rowKeyValue; });
            return __assign(__assign({}, state), { selectedRows: newSelectedRows });
        }
        case ActionType.UpdateSortDirection:
            var sortedColumns = getSortedColumns(columns, action.columnKey);
            return __assign(__assign({}, state), { columns: sortedColumns });
        case ActionType.UpdateVirtualScrolling:
            return __assign(__assign({}, state), { virtualScrolling: action.virtualScrolling });
        case ActionType.UpdateData:
            return __assign(__assign({}, state), { data: action.data });
        case ActionType.ScrollTable:
            if (virtualScrolling) {
                var scrollTop = action.scrollTop;
                return __assign(__assign({}, state), { virtualScrolling: __assign(__assign({}, virtualScrolling), { scrollTop: scrollTop }) });
            }
            break;
        case ActionType.UpdateGroupsExpanded: {
            var currentGroupsExpanded = groupsExpanded;
            if (!currentGroupsExpanded) {
                var preparedOptions = prepareTableOptions(state);
                currentGroupsExpanded = getExpandedGroups(preparedOptions.groupedData);
            }
            var newGroupsExpanded = updateExpandedGroups(currentGroupsExpanded, action.groupKey);
            return __assign(__assign({}, state), { groupsExpanded: newGroupsExpanded });
        }
    }
    return state;
};
export { kaReducer, };